package com.ruoyi.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.domain.*;
import com.ruoyi.mapper.*;
import com.ruoyi.service.*;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.web.request.*;
import com.ruoyi.web.response.TaskFindResponse;
import com.ruoyi.web.response.TaskGetInfoResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Objects;

@Transactional
@Service
public class TaskServiceImpl extends ServiceImpl<TaskMapper, Task> implements TaskService {

    @Autowired
    private TaskMapper taskMapper;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private StandardService standardService;

    @Autowired
    private TaskUserRelationService taskUserRelationService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private SystemReviewTaskService systemReviewTaskService;

    @Autowired
    private CarReviewTaskService carReviewTaskService;

    @Autowired
    private SystemReviewTaskMapper systemReviewTaskMapper;

    @Autowired
    private CarReviewTaskMapper carReviewTaskMapper;

    @Autowired
    private ReviewDetailsMapper reviewDetailsMapper;

    @Autowired
    private AutomobileEnterpriseMapper automobileEnterpriseMapper;

    @Autowired
    private TaskStateFacade taskStateFacade;

    @Autowired
    private TaskSampleRelationService taskSampleRelationService;

    @Autowired
    private SampleManagementMapper sampleManagementMapper;


    @Override
    public List<Task> findList(TaskListRequest request) {
        return taskMapper.findList(request);
    }

    @Override
    public void createTask(TaskCreateRequest request) {

        // 构建任务创建者
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser initiator = sysUserService.selectUserById(loginUser.getUserId());
        Standard standard = standardService.findListById(request.getStandardId());

        // 找到审查组长
        TaskUserRelation leader = null;
        List<TaskUserRelation> auditors = request.getAuditors();
        for(TaskUserRelation auditor : auditors) {
            if(auditor.getIsLeader() == 1) {
                leader = auditor;
                break;
            }
        }

        // 构建并保存任务信息
        // 总任务
        Task task = new Task();
        task.setTaskInitiatorId(loginUser.getUserId());
        task.setTaskInitiator(initiator.getNickName());
        task.setTaskInitiatorDept(initiator.getDept().getDeptName());
        task.setTaskStatus(Task.TASK_STATUS_NEW);
        task.setRunStatus(Task.RUN_STATUS_RUNNABLE);
        task.setLeaderId(leader.getUserId());
        task.setLeader(leader.getName());
        task.setCreateTime(new Date());
        task.setName(standard.getName());
        task.setStandardNo(standard.getStandardNo());
        task.setFile(standard.getFile());
        BeanUtils.copyBeanProp(task, request);

        if(request.getTaskList().contains(Task.SUB_TASK_GRFIS)) {
            // 保存车型审查问卷任务
            task.setCarReviewTaskId(saveCarReview(request,initiator,standard,leader));
        }
        if(request.getTaskList().contains(Task.SUB_TASK_SRAIF)) {
            // 保存体系审查任务
            task.setSystemReviewTaskId(saveSystemReview(request,initiator,standard,leader));
        }

        taskService.save(task);

        // 构建并保存该任务所关联的审核组信息
        for(TaskUserRelation auditor : auditors) {
            auditor.setTaskId(task.getId());
        }

        taskUserRelationService.saveBatch(auditors);

        // 保存关联的整车样品信息
        List<TaskSampleRelation> relations = request.getSample();
        if (relations.size() != 0 && relations != null) {
            for (TaskSampleRelation sampleRelation : relations) {
                sampleRelation.setTaskId(task.getId());
            }

            taskSampleRelationService.saveBatch(relations);
        }

        // 保存关联的零部件样品信息
        List<TaskSampleRelation> partRelations = request.getPartSample();
        if (partRelations.size() != 0 && partRelations != null) {
            for (TaskSampleRelation sampleRelation : partRelations) {
                sampleRelation.setTaskId(task.getId());
            }

            taskSampleRelationService.saveBatch(partRelations);
        }

        taskStateFacade.doCreate(task, request.getTaskList());

    }

    @Override
    public List<TaskFindResponse> findPending(TaskFindPendingRequest request) {
        List<TaskFindResponse> responses = taskMapper.findPending(request);
        for (TaskFindResponse response : responses) {
            if (response.getSystemReviewTaskId() != null) {
                response.setSystemReview(reviewDetailsMapper.count(response.getSystemReviewTaskId(),SystemReviewTask.TYPE));
            }
            if (response.getCarReviewTaskId() != null) {
                response.setCarReview(reviewDetailsMapper.count(response.getCarReviewTaskId(),CarReviewTask.TYPE));
            }
        }
        return responses;
    }

    @Override
    public List<Task> findFinish(TaskFindFinishRequest request) {
        return taskMapper.findFinish(request);
    }

    @Override
    public void editTask(TaskEditRequest request) {

        // 找到审查组长
        TaskUserRelation leader = null;
        List<TaskUserRelation> auditors = request.getAuditors();
        for(TaskUserRelation auditor : auditors) {
            if(auditor.getIsLeader() == 1) {
                leader = auditor;
                break;
            }
        }

        // 构建并保存任务信息
        // 总任务
        Task task = new Task();
        task.setLeaderId(leader.getUserId());
        task.setLeader(leader.getName());
        BeanUtils.copyBeanProp(task, request);

        taskService.updateById(task);

        // 构建并保存该任务所关联的审核组信息
        List<TaskUserRelation> list = taskUserRelationService.selectQTeamMembers(request.getId());
        taskUserRelationService.removeBatchByIds(list);
        for(TaskUserRelation auditor : auditors) {
            auditor.setTaskId(task.getId());
        }
        taskUserRelationService.saveBatch(auditors);

        // 保存关联的整车样品信息
        List<TaskSampleRelation> relationList = taskSampleRelationService.selectByTaskId(request.getId(),TaskSampleRelation.COMPLETE_VEHICLE_SAMPLE);
        taskSampleRelationService.removeBatchByIds(relationList);
        List<TaskSampleRelation> relations = request.getSample();
        if (relations.size() != 0 && relations != null) {
            for (TaskSampleRelation sampleRelation : relations) {
                sampleRelation.setTaskId(task.getId());
            }

            taskSampleRelationService.saveBatch(relations);
        }

        // 保存关联的零部件样品信息
        List<TaskSampleRelation> partRelationList = taskSampleRelationService.selectByTaskId(request.getId(),TaskSampleRelation.PART_VEHICLE_SAMPLE);
        taskSampleRelationService.removeBatchByIds(partRelationList);
        List<TaskSampleRelation> partRelations = request.getPartSample();
        if (partRelations.size() != 0 && partRelations != null) {
            for (TaskSampleRelation partSampleRelation : partRelations) {
                partSampleRelation.setTaskId(task.getId());
            }

            taskSampleRelationService.saveBatch(partRelations);
        }
    }

    @Override
    public void deleteTask(TaskDeleteRequest request) {
        Task task = taskMapper.selectById(request.getId());

        systemReviewTaskMapper.deleteById(task.getSystemReviewTaskId());
        carReviewTaskMapper.deleteById(task.getCarReviewTaskId());
        taskMapper.deleteById(request.getId());
    }

    @Override
    public void startTask(TaskStartRequest request) {
        Task task = taskMapper.selectById(request.getId());
        taskStateFacade.doStart(task);
    }

    @Override
    public List<TaskFindResponse> findCreate(TaskFindRequest request,Long userId) {
        List<TaskFindResponse> responses = taskMapper.findByUserId(request,userId);
        for (TaskFindResponse response : responses) {
            if (response.getSystemReviewTaskId() != null) {
                response.setSystemReview(reviewDetailsMapper.count(response.getSystemReviewTaskId(),SystemReviewTask.TYPE));
            }
            if (response.getCarReviewTaskId() != null) {
                response.setCarReview(reviewDetailsMapper.count(response.getCarReviewTaskId(),CarReviewTask.TYPE));
            }
        }
        return responses;
    }

    @Override
    public TaskGetInfoResponse getInfo(TaskGetInfoRequest request) {

        TaskGetInfoResponse response = new TaskGetInfoResponse();
        Task task = taskService.getById(request.getId());
        BeanUtils.copyBeanProp(response, task);
        // 小组成员
        List<TaskUserRelation> relation = taskUserRelationService.selectQTeamMembers(request.getId());
        response.setAuditors(relation);
        // 整车样品信息
        List<TaskSampleRelation> relationList = taskSampleRelationService.selectByTaskId(request.getId(),TaskSampleRelation.COMPLETE_VEHICLE_SAMPLE);
        if (relationList.size() != 0 && relationList != null) {
            List<Sample> completeVehicleSampleList = sampleManagementMapper.findByIdList(relationList);
            response.setCompleteVehicleSample(completeVehicleSampleList);
        }

        // 零部件样品信息
        List<TaskSampleRelation> partRelationList = taskSampleRelationService.selectByTaskId(request.getId(),TaskSampleRelation.PART_VEHICLE_SAMPLE);
        if (partRelationList.size() != 0 && partRelationList != null) {
            List<Sample> partVehicleSampleList = sampleManagementMapper.findByIdList(partRelationList);
            response.setPartVehicleSample(partVehicleSampleList);
        }

        return response;
    }

    @Override
    public List<Task> findUserFinish(TaskFindFinishRequest request,Long userId) {

        return taskMapper.findUserFinish(request,userId);
    }

    public Long saveSystemReview(TaskCreateRequest request,SysUser initiator,Standard standard,TaskUserRelation leader) {

        // 构建并保存任务信息
        SystemReviewTask task = new SystemReviewTask()
                .setTaskNo(request.getTaskNo())
                .setTaskInitiator(initiator.getNickName())
                .setTaskInitiatorDept(initiator.getDept().getDeptName())
                .setTaskStatus(SystemReviewTask.STATUS_NEW)
                .setLeaderId(leader.getUserId())
                .setLeader(leader.getName())
                .setCreateTime(new Date())
                .setStandardId(standard.getId())
                .setName(standard.getName())
                .setStandardNo(standard.getStandardNo())
                .setFile(standard.getFile());

        systemReviewTaskService.save(task);
        return task.getId();
    }

    public Long saveCarReview(TaskCreateRequest request,SysUser initiator,Standard standard,TaskUserRelation leader) {

        List<String> taskList = request.getTaskList();

        // 构建并保存任务信息
        CarReviewTask task = new CarReviewTask()
                .setTaskNo(request.getTaskNo())
                .setTaskInitiator(initiator.getNickName())
                .setTaskInitiatorDept(initiator.getDept().getDeptName())
                .setTaskStatus(CarReviewTask.TASK_STATUS_NEW)
                .setLeaderId(leader.getUserId())
                .setLeader(leader.getName())
                .setCreateTime(new Date())
                .setStandardId(standard.getId())
                .setName(standard.getName())
                .setStandardNo(standard.getStandardNo())
                .setFile(standard.getFile());

        carReviewTaskService.save(task);
        return task.getId();
    }


}
